var vueComposable = (function (exports, compositionApi) {
    'use strict';

    function unwrap(o) {
        return compositionApi.isRef(o) ? o.value : o;
    }
    function wrap(o) {
        return compositionApi.isRef(o) ? o : compositionApi.ref(o);
    }
    const isArray = Array.isArray;
    const isFunction = (val) => typeof val === "function";
    const isString = (val) => typeof val === "string";
    const isSymbol = (val) => typeof val === "symbol";
    const isBoolean = (val) => typeof val === "boolean";
    const isDate = (val) => isObject(val) && isFunction(val.getTime);
    const isNumber = (val) => typeof val === "number";
    const isObject = (val) => val !== null && typeof val === "object";
    const isElement = (val) => isObject(val) && !!val.tagName;
    function isPromise(val) {
        return isObject(val) && isFunction(val.then) && isFunction(val.catch);
    }
    const NO_OP = () => { };
    const FALSE_OP = () => false;
    function promisedTimeout(timeout) {
        return new Promise(res => {
            setTimeout(res, timeout);
        });
    }
    function minMax(val, min, max) {
        if (val < min)
            return min;
        if (val > max)
            return max;
        return val;
    }

    function usePromise(fn, throwException = false) {
        if (!fn) {
            throw new Error(`[usePromise] argument can't be '${fn}'`);
        }
        if (typeof fn !== "function") {
            throw new Error(`[usePromise] expects function, but received ${typeof fn}`);
        }
        const loading = compositionApi.ref(false);
        const error = compositionApi.ref(null);
        const result = compositionApi.ref(null);
        const promise = compositionApi.ref();
        const exec = async (...args) => {
            loading.value = true;
            error.value = null;
            result.value = null;
            let throwExp = args && fn.length !== args.length && args.length > 0 && isBoolean(args[args.length - 1]) ? args[args.length - 1] : throwException;
            const currentPromise = (promise.value = fn(...args));
            try {
                const r = await currentPromise;
                if (promise.value === currentPromise) {
                    result.value = r;
                }
                return r;
            }
            catch (er) {
                if (promise.value === currentPromise) {
                    error.value = er;
                    result.value = null;
                }
                return throwExp ? currentPromise : undefined;
            }
            finally {
                if (promise.value === currentPromise) {
                    loading.value = false;
                }
            }
        };
        return {
            exec,
            result,
            promise,
            loading,
            error
        };
    }

    function useCancellablePromise(fn, throwException = false) {
        const cancelled = compositionApi.ref(false);
        let _cancel = undefined;
        const cancel = (result) => _cancel(result); // TODO add warning if cancel is undefined
        const promise = (p) => new Promise((res, rej) => {
            _cancel = result => {
                cancelled.value = true;
                rej(result);
            };
            p.then(res).catch(rej);
        });
        const use = usePromise((...args) => promise(fn(...args)), throwException);
        return {
            ...use,
            cancel,
            cancelled
        };
    }

    const MAX_RETRIES = 9000;
    /* istanbul ignore next */
    const ExecutionId = Symbol( "RetryId" );
    /* istanbul ignore next */
    const CancellationToken = Symbol( "CancellationToken" );
    const defaultStrategy = async (options, context, factory, args) => {
        const retryId = context[ExecutionId].value;
        let i = -1;
        const maxRetries = options.maxRetries || MAX_RETRIES + 1;
        const delay = options.retryDelay || noDelay;
        context.retryErrors.value = [];
        context.isRetrying.value = false;
        context.nextRetry.value = undefined;
        let nextRetry = undefined;
        do {
            let success = false;
            let result = null;
            try {
                ++i;
                if (args) {
                    result = factory(...args);
                }
                else {
                    result = factory();
                }
                if (isPromise(result)) {
                    result = await result;
                }
                // is cancelled?
                if (context[CancellationToken].value) {
                    return null;
                }
                success = true;
            }
            catch (error) {
                result = null;
                context.retryErrors.value.push(error);
            }
            // is our retry current one?
            if (retryId !== context[ExecutionId].value) {
                return result;
            }
            if (success) {
                context.isRetrying.value = false;
                context.nextRetry.value = undefined;
                return result;
            }
            if (i >= maxRetries) {
                context.isRetrying.value = false;
                context.nextRetry.value = undefined;
                return Promise.reject(new Error(`[useRetry] max retries reached ${maxRetries}`));
            }
            context.isRetrying.value = true;
            const now = Date.now();
            const pDelayBy = delay(i); // wrapped
            const delayBy = isPromise(pDelayBy) ? await pDelayBy : pDelayBy; // unwrap promise
            if (!isPromise(pDelayBy) || !!delayBy) {
                if (isNumber(delayBy)) {
                    nextRetry = delayBy;
                }
                else if (isDate(delayBy)) {
                    nextRetry = delayBy.getTime();
                }
                else {
                    throw new Error(`[useRetry] invalid value received from options.retryDelay '${typeof delayBy}'`);
                }
                // if the retry is in the past, means we need to await that amount
                if (nextRetry < now) {
                    context.nextRetry.value = now + nextRetry;
                }
                else {
                    context.nextRetry.value = nextRetry;
                    nextRetry = nextRetry - now;
                }
                if (nextRetry > 0) {
                    await promisedTimeout(nextRetry);
                }
            }
            // is cancelled?
            if (context[CancellationToken].value) {
                return null;
            }
            // is our retry current one?
            if (retryId !== context[ExecutionId].value) {
                return result;
            }
        } while (i < MAX_RETRIES);
        return null;
    };
    function useRetry(options, factory) {
        const opt = !options || isFunction(options) ? {} : options;
        const fn = isFunction(options) ? options : factory;
        if (!isFunction(options) && !isObject(options)) {
            throw new Error("[useRetry] options needs to be 'object'");
        }
        if (!!fn && !isFunction(fn)) {
            throw new Error("[useRetry] factory needs to be 'function'");
        }
        const isRetrying = compositionApi.ref(false);
        const nextRetry = compositionApi.ref();
        const retryErrors = compositionApi.ref([]);
        const cancellationToken = { value: false };
        const retryId = { value: 0 };
        const retryCount = compositionApi.computed(() => retryErrors.value.length);
        const context = {
            isRetrying,
            retryCount,
            nextRetry,
            retryErrors,
            [ExecutionId]: retryId,
            [CancellationToken]: cancellationToken
        };
        const exec = fn
            ? (...args) => {
                ++context[ExecutionId].value;
                return defaultStrategy(opt, context, fn, args);
            }
            : (f) => {
                ++context[ExecutionId].value;
                return defaultStrategy(opt, context, f, undefined);
            };
        const cancel = () => {
            context.isRetrying.value = false;
            context.retryErrors.value.push(new Error("[useRetry] cancelled"));
            context.nextRetry.value = undefined;
            cancellationToken.value = true;
        };
        return {
            ...context,
            cancel,
            exec
        };
    }
    const exponentialDelay = retryNumber => {
        const delay = Math.pow(2, retryNumber) * 100;
        const randomSum = delay * 0.2 * Math.random(); // 0-20% of the delay
        return delay + randomSum;
    };
    const noDelay = () => 0;

    function useDebounce(handler, wait, options) {
        return debounce(handler, wait, options);
    }
    function debounce(func, waitMilliseconds = 50, options = {
        isImmediate: false
    }) {
        let timeoutId;
        return function (...args) {
            const context = this;
            const doLater = function () {
                timeoutId = undefined;
                if (!options.isImmediate) {
                    func.apply(context, args);
                }
            };
            const shouldCallNow = options.isImmediate && timeoutId === undefined;
            if (timeoutId !== undefined) {
                clearTimeout(timeoutId);
            }
            timeoutId = setTimeout(doLater, waitMilliseconds);
            if (shouldCallNow) {
                func.apply(context, args);
            }
        };
    }

    function usePagination(options) {
        const _currentPage = wrap(options.currentPage);
        const _pageSize = wrap(options.pageSize);
        const _offset = compositionApi.ref(0);
        const total = wrap(options.total);
        const offset = compositionApi.computed({
            get() {
                return _offset.value;
            },
            set(v) {
                if (typeof v !== "number") {
                    /* istanbul ignore else */
                    {
                        console.warn(`[offset] expected number but got: '${typeof v}' value: '${v}'`);
                    }
                    return;
                }
                _offset.value = Math.min(v, total.value);
            }
        });
        const currentPage = compositionApi.computed({
            get() {
                return _currentPage.value;
            },
            set(v) {
                if (typeof v !== "number") {
                    /* istanbul ignore else  */
                    {
                        console.warn(`[currentPage] expected number but got: '${typeof v}' value: '${v}'`);
                    }
                    return;
                }
                _currentPage.value = minMax(v, 1, lastPage.value);
                // set the offset
                offset.value = (_currentPage.value - 1) * pageSize.value;
            }
        });
        const pageSize = compositionApi.computed({
            get() {
                return _pageSize.value;
            },
            set(v) {
                if (typeof v !== "number") {
                    /* istanbul ignore else */
                    {
                        console.warn(`[pageSize] expected number but got: '${typeof v}' value: '${v}'`);
                    }
                    return;
                }
                _pageSize.value = v;
            }
        });
        const lastPage = compositionApi.computed(() => Math.ceil(total.value / pageSize.value));
        // make sure the current page is the correct value
        currentPage.value = _currentPage.value;
        const prev = () => --currentPage.value;
        const next = () => ++currentPage.value;
        const first = () => (currentPage.value = 1);
        const last = () => (currentPage.value = lastPage.value);
        compositionApi.watch([total, pageSize], () => {
            if (currentPage.value > lastPage.value) {
                currentPage.value = lastPage.value;
            }
        }, { lazy: true } // no need to run on first render
        );
        return {
            // Mutable state
            pageSize,
            total,
            currentPage,
            offset,
            // Computed
            lastPage,
            // Functions
            next,
            prev,
            first,
            last
        };
    }

    function useArrayPagination(array, options) {
        const arrayRef = wrap(array);
        const pagination = usePagination({
            ...{
                currentPage: 1,
                pageSize: 10,
            },
            ...options,
            total: compositionApi.computed(() => arrayRef.value.length)
        });
        const result = compositionApi.computed(() => {
            const array = arrayRef.value;
            if (!Array.isArray(array))
                return [];
            return array.slice(pagination.offset.value, pagination.offset.value + pagination.pageSize.value);
        });
        return {
            ...pagination,
            result
        };
    }

    function useEvent(el, name, listener, options) {
        const element = wrap(el);
        const remove = () => element.value.removeEventListener(name, listener);
        compositionApi.onMounted(() => element.value.addEventListener(name, listener, options));
        compositionApi.onUnmounted(remove);
        return remove;
    }

    function useOnMouseMove(el, options, wait) {
        const mouseX = compositionApi.ref(0);
        const mouseY = compositionApi.ref(0);
        let handler = (ev) => {
            mouseX.value = ev.x;
            mouseY.value = ev.y;
        };
        const eventOptions = typeof options === "number" ? undefined : options;
        const ms = typeof options === "number" ? options : wait;
        if (ms) {
            handler = useDebounce(handler, wait);
        }
        const remove = useEvent(el, "mousemove", handler, eventOptions);
        return {
            mouseX,
            mouseY,
            remove
        };
    }

    function useOnResize(el, options, wait) {
        const element = wrap(el);
        const height = compositionApi.ref(element.value && element.value.clientHeight);
        const width = compositionApi.ref(element.value && element.value.clientWidth);
        let handler = () => {
            height.value = element.value.clientHeight;
            width.value = element.value.clientWidth;
        };
        const eventOptions = typeof options === "number" ? undefined : options;
        const ms = typeof options === "number" ? options : wait;
        if (ms) {
            handler = useDebounce(handler, wait);
        }
        // resize seems only to be fired against the window
        const remove = useEvent(window, "resize", handler, eventOptions || { passive: true });
        return {
            height,
            width,
            remove
        };
    }

    function useOnScroll(el, options, wait) {
        const element = wrap(el);
        const scrollTop = compositionApi.ref(element.value && element.value.scrollTop);
        const scrollLeft = compositionApi.ref(element.value && element.value.scrollLeft);
        let handler = (ev) => {
            scrollTop.value = element.value.scrollTop;
            scrollLeft.value = element.value.scrollLeft;
        };
        const eventOptions = typeof options === "number" ? undefined : options;
        const ms = typeof options === "number" ? options : wait;
        if (ms) {
            handler = useDebounce(handler, wait);
        }
        const remove = useEvent(element, "scroll", handler, eventOptions);
        return {
            scrollTop,
            scrollLeft,
            remove
        };
    }

    function useFetch(options) {
        const json = compositionApi.ref(null);
        // TODO add text = ref<string> ??
        const jsonError = compositionApi.ref(null);
        const isJson = options ? options.isJson !== false : true;
        const parseImmediate = options ? options.parseImmediate !== false : true;
        const use = usePromise(async (request, init) => {
            const response = await fetch(request, init);
            if (isJson) {
                const pJson = response
                    .json()
                    .then(x => (json.value = x))
                    .catch(x => {
                    json.value = null;
                    jsonError.value = x;
                });
                if (parseImmediate) {
                    await pJson;
                }
            }
            return response;
        });
        const status = compositionApi.computed(() => (use.result.value && use.result.value.status) || null);
        const statusText = compositionApi.computed(() => (use.result.value && use.result.value.statusText) || null);
        return {
            ...use,
            json,
            jsonError,
            status,
            statusText
        };
    }

    function useWebSocket(url, protocols) {
        const ws = new WebSocket(url, protocols);
        const messageEvent = compositionApi.ref(null);
        const errorEvent = compositionApi.ref();
        const data = compositionApi.ref(null);
        const isOpen = compositionApi.ref(false);
        const isClosed = compositionApi.ref(false);
        const errored = compositionApi.ref(false);
        /* istanbul ignore next  */
        let lastMessage = ( Date.now()) || undefined;
        ws.addEventListener("message", x => {
            messageEvent.value = x;
            data.value = x.data;
            // if the messages are to quick, we need to warn
            /* istanbul ignore else  */
            {
                if (Date.now() - lastMessage < 2) {
                    console.warn('[useWebSocket] message rate is too high, if you are using "data" or "messageEvent"' +
                        " you might not get updated of all the messages." +
                        ' Use "ws..addEventListener("message", handler)" instead');
                }
                lastMessage = Date.now();
            }
        });
        ws.addEventListener("error", error => {
            errorEvent.value = error;
            errored.value = true;
        });
        ws.addEventListener("close", () => {
            isOpen.value = false;
            isClosed.value = true;
        });
        ws.addEventListener("open", () => {
            isOpen.value = true;
            isClosed.value = false;
        });
        const send = (data) => ws.send(data);
        const close = (code, reason) => {
            ws.close(code, reason);
        };
        return {
            ws,
            send,
            close,
            messageEvent,
            errorEvent,
            data,
            isOpen,
            isClosed,
            errored
        };
    }

    function useIntersectionObserver(refEl, refOptions) {
        const wrappedElement = refEl ? wrap(refEl) : undefined;
        const element = wrappedElement && (isElement(wrappedElement.value) || !wrappedElement.value)
            ? wrappedElement
            : undefined;
        const options = compositionApi.computed(() => refOptions
            ? unwrap(refOptions)
            : !element
                ? unwrap(refEl)
                : undefined);
        const elements = compositionApi.ref(element && element.value ? [element.value] : []);
        const isIntersecting = compositionApi.computed(() => elements.value.length > 0 && elements.value.every(x => x.isIntersecting));
        const handling = (entries) => {
            elements.value = entries;
        };
        let observer = compositionApi.ref();
        compositionApi.watch(options, options => {
            if (observer.value) {
                observer.value.disconnect();
            }
            const opts = (options &&
                options && {
                root: unwrap(options.root),
                rootMargin: unwrap(options.rootMargin),
                threshold: unwrap(options.threshold)
            }) ||
                undefined;
            observer.value = new IntersectionObserver(handling, opts);
            const targets = elements.value.map(x => x.target);
            targets.forEach(observer.value.observe);
        }, { deep: true });
        const observe = (element) => {
            const e = unwrap(element);
            observer.value.observe(e);
        };
        const unobserve = (element) => {
            const e = unwrap(element);
            observer.value.unobserve(e);
        };
        const disconnect = () => observer.value.disconnect();
        // if the element is passed we should add hooks
        if (element) {
            // if value is defined it is already being observed
            if (!element.value) {
                compositionApi.onMounted(() => {
                    if (element.value) {
                        observe(element);
                    }
                });
            }
            compositionApi.onUnmounted(() => {
                disconnect();
            });
        }
        // // debug is still work in progress, would be nice to provide some
        // // information about the target
        // /* istanbul ignore next */
        // const debug = () => {
        //   if (elements.value.length === 0) {
        //     (true !== 'production') && console.warn('[IntersectionObserver] no elements provided, did you mount the component?')
        //     return;
        //   }
        //   // TODO: add border to the elements 
        // };
        return {
            elements,
            observe,
            unobserve,
            disconnect,
            isIntersecting
        };
    }

    function useNetworkInformation() {
        const connection = navigator.connection ||
            navigator.mozConnection ||
            navigator.webkitConnection;
        const supported = compositionApi.computed(() => !!connection);
        const downlink = compositionApi.ref(0);
        const downlinkMax = compositionApi.ref(0);
        const effectiveType = compositionApi.ref("unknown");
        const rtt = compositionApi.ref(0);
        const saveData = compositionApi.ref(false);
        const type = compositionApi.ref("none");
        let handler = NO_OP;
        let remove = NO_OP;
        /* istanbul ignore else  */
        if (connection) {
            handler = () => {
                downlink.value = connection.downlink;
                downlinkMax.value = connection.downlinkMax;
                effectiveType.value = connection.effectiveType;
                rtt.value = connection.rtt;
                saveData.value = connection.saveData;
                type.value = connection.type;
            };
            remove = useEvent(connection, "change", handler, { passive: true });
            handler();
        }
        else {
            /* istanbul ignore else */
            {
                console.warn("[navigator.connection] not found, networkInformation not available.");
            }
        }
        return {
            supported,
            downlink,
            downlinkMax,
            effectiveType,
            rtt,
            saveData,
            type,
            remove
        };
    }

    let online = undefined;
    function useOnline() {
        const supported = "onLine" in navigator;
        // not sure how to test this :/
        if (!supported) {
            online = compositionApi.ref(false);
        }
        if (!online) {
            online = compositionApi.ref(navigator.onLine);
            window.addEventListener("offline", () => (online.value = false), {
                passive: true
            });
            window.addEventListener("online", () => (online.value = true), {
                passive: true
            });
        }
        return {
            supported,
            online
        };
    }

    let visibility = undefined;
    let hidden = undefined;
    function usePageVisibility() {
        if (!hidden) {
            hidden = compositionApi.ref(document.hidden);
        }
        if (!visibility) {
            visibility = compositionApi.ref(document.visibilityState);
            document.addEventListener("visibilitychange", () => {
                visibility.value = document.visibilityState;
                hidden.value = document.hidden;
            }, { passive: true }
            // true
            );
        }
        return {
            visibility,
            hidden
        };
    }

    let language = undefined;
    let languages = undefined;
    function useLanguage() {
        if (!language) {
            language = compositionApi.ref(navigator.language);
        }
        if (!languages) {
            languages = compositionApi.ref(navigator.languages);
            const change = () => {
                language.value = navigator.language;
                languages.value = navigator.languages;
            };
            window.addEventListener('languagechange', change, { passive: true });
        }
        return {
            language,
            languages
        };
    }

    function useMatchMedia(query) {
        const mediaQueryList = compositionApi.ref(matchMedia(query));
        const matches = compositionApi.ref(mediaQueryList.value.matches);
        const process = (e) => {
            matches.value = e.matches;
        };
        mediaQueryList.value.addEventListener("change", process, { passive: true });
        const remove = () => mediaQueryList.value.removeEventListener("change", process);
        compositionApi.onUnmounted(remove);
        return {
            mediaQueryList,
            remove,
            matches
        };
    }

    function useBreakpoint(breakpoints) {
        const result = {};
        const map = new Map();
        const current = compositionApi.ref();
        let sorted = [];
        const removeMedia = [];
        for (const key in breakpoints) {
            const bp = breakpoints[key];
            if (isNumber(bp)) {
                const r = compositionApi.ref(false);
                result[key] = r;
                map.set(bp, {
                    name: key,
                    valid: r
                });
                sorted.push(bp);
            }
            else {
                const { matches, remove } = useMatchMedia(bp);
                result[key] = matches;
                removeMedia.push(remove);
            }
        }
        sorted = sorted.sort((a, b) => b - a);
        const resize = () => {
            const width = window.innerWidth;
            let c = undefined;
            for (let i = 0; i < sorted.length; i++) {
                const bp = sorted[i];
                const r = map.get(bp);
                r.valid.value = width >= bp;
                if (width >= bp && c === undefined) {
                    c = r.name;
                }
            }
            current.value = c;
        };
        const processResize = useDebounce(resize, 10);
        const remove = () => window.removeEventListener("resize", processResize);
        compositionApi.onMounted(() => {
            resize();
            window.addEventListener("resize", processResize, {
                passive: true
            });
        });
        compositionApi.onUnmounted(() => {
            remove();
            removeMedia.forEach(x => x());
        });
        return {
            ...result,
            remove,
            current
        };
    }

    /* istanbul ignore next */
    function isQuotaExceededError(e, storage) {
        return e instanceof DOMException && (
        // everything except Firefox
        e.code === 22 ||
            // Firefox
            e.code === 1014 ||
            // test name field too, because code might not be present
            // everything except Firefox
            e.name === 'QuotaExceededError' ||
            // Firefox
            e.name === 'NS_ERROR_DOM_QUOTA_REACHED') &&
            // acknowledge QuotaExceededError only if there's something already stored
            (storage && storage.length !== 0);
    }
    // based on https://developer.mozilla.org/en-US/docs/Web/API/Web_Storage_API/Using_the_Web_Storage_API
    function storageAvailable(storage) {
        try {
            const x = '__storage_test__';
            storage.setItem(x, x);
            storage.removeItem(x);
            return true;
        }
        catch (e) {
            return isQuotaExceededError(e, storage);
        }
    }
    function safeParse(serializer, value) {
        try {
            return serializer.parse(value);
        }
        catch {
            return value;
        }
    }
    let storageMap = undefined;
    function useWebStorage(type, serializer = JSON, ms = 10) {
        const storage = window[type];
        const supported = storageAvailable(storage);
        const remove = () => storageMap.delete(type);
        if (!storageMap) {
            storageMap = new Map();
            window.addEventListener('storage', (e) => {
                if (e.newValue === e.oldValue) {
                    return;
                }
                let webStore = storageMap.get('localStorage');
                if (e.storageArea === window.localStorage) {
                    webStore = storageMap.get('localStorage');
                }
                else {
                    webStore = storageMap.get('sessionStorage');
                }
                if (webStore && Object.keys(webStore.$syncKeys).length > 0) {
                    if (e.key === null) {
                        webStore.clear();
                    }
                    else if (webStore.$syncKeys[e.key]) {
                        if (e.newValue === null) {
                            webStore.removeItem(e.key);
                        }
                        else {
                            webStore.updateItem(e.key, e.newValue);
                        }
                    }
                }
            });
        }
        let store = storageMap.get(type);
        let quotaError;
        if (supported) {
            if (!store) {
                quotaError = compositionApi.ref(false);
                store = {
                    $refMap: new Map(),
                    $watchHandlers: new Map(),
                    $syncKeys: {},
                    $quotaError: quotaError,
                    key: storage.key,
                    length: storage.length,
                    setSync(key, sync) {
                        if (sync) {
                            this.$syncKeys[key] = true;
                        }
                        else {
                            delete this.$syncKeys[key];
                        }
                    },
                    clear() {
                        this.$refMap.forEach((_, k) => this.removeItem(k));
                    },
                    removeItem(k) {
                        const item = this.$refMap.get(k);
                        // remove the object value if item deleted
                        if (item) {
                            item.value = undefined;
                        }
                        // clear the watch
                        const stop = this.$watchHandlers.get(k);
                        if (stop) {
                            stop();
                        }
                        delete this.$syncKeys[k];
                        this.$refMap.delete(k);
                        storage.removeItem(k);
                    },
                    getItem(k) {
                        let r = this.$refMap.get(k);
                        if (r) {
                            return r;
                        }
                        const data = storage.getItem(k);
                        if (!data) {
                            return null;
                        }
                        return this.setItem(k, safeParse(serializer, data));
                    },
                    setItem(k, v) {
                        const reference = wrap(v);
                        this.$refMap.set(k, reference);
                        const save = (key, value) => {
                            try {
                                const data = isString(value) ? value : serializer.stringify(value);
                                storage.setItem(key, data);
                            }
                            catch (e) {
                                quotaError.value = isQuotaExceededError(e, storage);
                            }
                        };
                        save(k, v);
                        const stop = compositionApi.watch(reference, debounce((r) => {
                            save(k, r);
                        }, ms), {
                            lazy: true,
                            deep: true
                        });
                        this.$watchHandlers.set(k, stop);
                        return reference;
                    },
                    updateItem(k, data) {
                        let r = this.$refMap.get(k);
                        if (r) {
                            r.value = safeParse(serializer, data);
                        }
                    }
                };
                storageMap.set(type, store);
            }
            else {
                quotaError = store.$quotaError;
            }
        }
        else {
            quotaError = compositionApi.ref(false);
            store = {};
        }
        return {
            supported,
            quotaError,
            store,
            remove
        };
    }

    function useLocalStorage(key, defaultValue) {
        const { supported, store } = useWebStorage('localStorage');
        let remove = NO_OP;
        let clear = NO_OP;
        let setSync = NO_OP;
        let storage = undefined;
        if (supported && store) {
            setSync = (s) => store.setSync(key, s);
            remove = () => store.removeItem(key);
            clear = () => store.clear();
            storage = store.getItem(key);
            if (!storage) {
                storage = store.setItem(key, defaultValue);
            }
        }
        else {
            /* istanbul ignore else */
            {
                console.warn('[localStorage] is not available');
            }
        }
        return {
            supported,
            storage,
            clear,
            remove,
            setSync,
        };
    }

    function useSessionStorage(key, defaultValue) {
        const { supported, store } = useWebStorage('sessionStorage');
        let remove = NO_OP;
        let clear = NO_OP;
        let setSync = FALSE_OP;
        let storage = undefined;
        if (supported && store) {
            /* istanbul ignore else */
            {
                setSync = () => console.warn('sync is not supported, please `useLocalStorage` instead');
            }
            remove = () => store.removeItem(key);
            clear = () => store.clear();
            storage = store.getItem(key);
            if (!storage) {
                storage = store.setItem(key, defaultValue);
            }
        }
        else {
            /* istanbul ignore else */
            {
                console.warn('[sessionStorage] is not available');
            }
        }
        return {
            supported,
            storage,
            clear,
            remove,
            setSync
        };
    }

    let canUseLocalStorage = undefined;
    function useStorage(key, defaultValue) {
        if (canUseLocalStorage === undefined) {
            canUseLocalStorage = useWebStorage('localStorage').supported;
        }
        return canUseLocalStorage
            ? useLocalStorage(key, defaultValue)
            : useSessionStorage(key, defaultValue);
    }

    exports.FALSE_OP = FALSE_OP;
    exports.NO_OP = NO_OP;
    exports.debounce = debounce;
    exports.exponentialDelay = exponentialDelay;
    exports.isArray = isArray;
    exports.isBoolean = isBoolean;
    exports.isDate = isDate;
    exports.isElement = isElement;
    exports.isFunction = isFunction;
    exports.isNumber = isNumber;
    exports.isObject = isObject;
    exports.isPromise = isPromise;
    exports.isString = isString;
    exports.isSymbol = isSymbol;
    exports.minMax = minMax;
    exports.noDelay = noDelay;
    exports.promisedTimeout = promisedTimeout;
    exports.storageAvailable = storageAvailable;
    exports.unwrap = unwrap;
    exports.useArrayPagination = useArrayPagination;
    exports.useBreakpoint = useBreakpoint;
    exports.useCancellablePromise = useCancellablePromise;
    exports.useDebounce = useDebounce;
    exports.useEvent = useEvent;
    exports.useFetch = useFetch;
    exports.useIntersectionObserver = useIntersectionObserver;
    exports.useLanguage = useLanguage;
    exports.useLocalStorage = useLocalStorage;
    exports.useMatchMedia = useMatchMedia;
    exports.useNetworkInformation = useNetworkInformation;
    exports.useOnMouseMove = useOnMouseMove;
    exports.useOnResize = useOnResize;
    exports.useOnScroll = useOnScroll;
    exports.useOnline = useOnline;
    exports.usePageVisibility = usePageVisibility;
    exports.usePagination = usePagination;
    exports.usePromise = usePromise;
    exports.useRetry = useRetry;
    exports.useSessionStorage = useSessionStorage;
    exports.useStorage = useStorage;
    exports.useWebSocket = useWebSocket;
    exports.useWebStorage = useWebStorage;
    exports.wrap = wrap;

    return exports;

}({}, vueCompositionApi));
